#ifndef ENCRYPTION_H
#define ENCRYPTION_H
#endif


#include "zlib.h"


// Packet encryption

void _cdecl BotSession::InitializeEncryption(DWORD Key)
{
	if (SentKey == Key)
	{
		SSEncrTable.SSKey = 0;
		return;
	}

	SSET * table = &SSEncrTable;

	SSEncrTable.SSKey = Key;

	// Do the ASM thing (Assembly rips provided by Coconut Emulator)
	__asm
	{
		// Initialize calculations
		mov ebx, table				// ebx = ptr to SSTable struct
		mov ecx, [ebx]				// ecx = value of SSTable.SSKey
		mov edi, ebx				// edi = ptr to SSTable.SSBuf
		add edi, 4					//    ''
		mov esi, 104h				// esi = LoopCounter (SSTable.SSBuf size=208h by)

SSInit_LoopHead:
		mov eax, 834E0B5Fh			// Calculations
		imul ecx					//   ''
		add edx, ecx				//   ''
		sar edx, 10h				//   ''
		mov eax, edx				//   ''
		shr eax, 1Fh				//   ''
		add edx, eax				//   ''
		lea eax, [edx+8*edx]		//   ''
		shl eax, 3					//   ''
		sub eax, edx				//   ''
		lea eax, [eax+4*eax]		//   ''
		shl eax, 1					//   ''
		sub eax, edx				//   ''
		shl eax, 2					//   ''
		mov ebx, eax				//   ''
		mov eax, ecx				//   ''
		cdq							//   ''
		mov ecx, 1F31Dh				//   ''
		idiv ecx					//   ''
		imul edx, 41A7h				//   ''
		sub edx, ebx				//   ''
		add edx, 7Bh				//   ''
		test edx, edx				//   ''
		mov ecx, edx				//   ''
		jg SSInit_Skip				//   ''
		lea ecx, [edx+7FFFFFFFh]	//   ''

SSInit_Skip:
		mov [edi], cx				// Save 2 by to SSTable.SSBuf
		add edi, 2					// Point to next position in SSTable.SSBuf
		dec esi						// LoopCounter = LoopCounter - 1
		jnz SSInit_LoopHead			// End loop (loop until LoopCounter=0)
	};
}

void BotSession::Encrypt(BYTE * Message, DWORD Length)
{
	if (SSEncrTable.SSKey == 0) return;

	LONG IV = SSEncrTable.SSKey;
	BYTE buffer[520];
	memset(buffer, 0, 520);

	if (Message[0] == 0)
		memcpy(buffer, &Message[2], Length - 2);
	else
		memcpy(buffer, &Message[1], Length - 1);

	for (SHORT Counter = 0; Counter < 520; Counter += 4)
	{
		IV = GetLong(buffer, Counter) ^ GetLong(SSEncrTable.SSBuf, Counter) ^ IV;

		*(long*)&buffer[Counter] = IV;
	}

	if (Message[0] == 0)
		memcpy(&Message[2], buffer, Length - 2);
	else
		memcpy(&Message[1], buffer, Length - 1);
}

void BotSession::Decrypt(BYTE * Message, DWORD Length)
{
	if (SSEncrTable.SSKey == 0) return;

	LONG IV = SSEncrTable.SSKey;
	LONG ESI;
	LONG EDX;
	BYTE buffer[520];
	memset(buffer, 0, 520);

	if (Message[0] == 0)
		memcpy(buffer, &Message[2], Length - 2);
	else
		memcpy(buffer, &Message[1], Length - 1);

	for (SHORT Counter = 0; Counter < 520; Counter += 4)
	{
		EDX = GetLong(buffer, Counter);
		ESI = GetLong(SSEncrTable.SSBuf, Counter) ^ IV ^ EDX;
		IV = EDX;
		*(long*)&buffer[Counter] = ESI;
	}

	if (Message[0] == 0)
		memcpy(&Message[2], buffer, Length - 2);
	else
		memcpy(&Message[1], buffer, Length - 1);
}


// Billing Server password encryption

void HashPassword(BYTE * Password)
{
	size_t StrLen = strlen((char*)Password);
	BYTE Factor = SimpleChecksum(Password, StrLen);
	BYTE Char;

	for (DWORD L = 0; L < StrLen; L++)
	{
		Char = Password[L] ^ Factor;
		Factor = (Factor ^ (Char << (Char & 3))) & 255;
		if (Char == 0)
			Char = 0xED;
		Password[L] = Char;
	}
}

void InverseHash(size_t StrLen, BYTE * In, BYTE * Out, BYTE Key)
{
	for (DWORD L = 0; L < StrLen; L++)
	{
		BYTE Char = In[L];
		if (Char == 0xED)
			Char = 0;
		Out[L] = Char ^ (BYTE)Key;
		Key = (Key ^ (Char << (Char & 3))) & 255;
	}
}

bool InvalidPasswordCharacter(BYTE c)
{
	if ((c < 33) || (c > 126))
		return true;

	return false;
}

void DecryptHashedPassword(BYTE * Password)
{
	size_t StrLen = strlen((char*)Password);

	BYTE * Buffer = new BYTE[StrLen + 1];
	Buffer[StrLen] = 0;

	// Passwords of EVEN length are very easy to crack.
	if (StrLen % 2 != 0)
	{
		// Guess and check to find one of the solutions
		for (int i = 0; i < 256; i++)
		{
			// Generate a possible solution
			InverseHash(StrLen, Password, Buffer, i);

			// Compare resultant hash with given hash
			BYTE Char, Key = i;
			bool OK = true;

			for (DWORD L = 0; L < StrLen; L++)
			{
				Char = Buffer[L];

				if (InvalidPasswordCharacter(Char))
				{
					OK = false;

					break;
				}

				Char ^= Key;
				Key = (Key ^ (Char << (Char & 3))) & 255;
				if (Char == 0)
					Char = 0xED;

				if (Char != Password[L])
				{
					OK = false;

					break;
				}
			}

			if (OK)
				break;
		}
	}
	else
	{
		// Generate password checksum
		InverseHash(StrLen, Password, Buffer, 0);

		// Generate actual password
		InverseHash(StrLen, Password, Buffer, SimpleChecksum(Buffer, StrLen));
	}

	memcpy(Password, Buffer, StrLen);

	delete Buffer;
}


// Security Checksum

LONG OneTimePad[1024] = {0x00000000, 0x77073096, 0xee0e612c, 0x990951ba, 
                         0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3, 
                         0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988, 
                         0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91, 
                         0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de, 
                         0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7, 
                         0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec, 
                         0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5, 
                         0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172, 
                         0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b, 
                         0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940, 
                         0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59, 
                         0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116, 
                         0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f, 
                         0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924, 
                         0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d, 
                         0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a, 
                         0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433, 
                         0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818, 
                         0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01, 
                         0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e, 
                         0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457, 
                         0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c, 
                         0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65, 
                         0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2, 
                         0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb, 
                         0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0, 
                         0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9, 
                         0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086, 
                         0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f, 
                         0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4, 
                         0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad, 
                         0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a, 
                         0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683, 
                         0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8, 
                         0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1, 
                         0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe, 
                         0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7, 
                         0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc, 
                         0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5, 
                         0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252, 
                         0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b, 
                         0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60, 
                         0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79, 
                         0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236, 
                         0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f, 
                         0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04, 
                         0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d, 
                         0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a, 
                         0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713, 
                         0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38, 
                         0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21, 
                         0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e, 
                         0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777, 
                         0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c, 
                         0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45, 
                         0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2, 
                         0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db, 
                         0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0, 
                         0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9, 
                         0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6, 
                         0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf, 
                         0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94, 
                         0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d};

LONG GetFileChecksum(BYTE * FileContents, LONG Length)
{
	unsigned long Index = 0,
				  Key   = 0xFFFFFFFF;

	for (int i = 0; i < Length; i++)
	{
		Index = (Key & 0xFF) ^ FileContents[i];
		Key >>= 8;
		Index = OneTimePad[Index];
		Key  ^= Index;
	}

	return (Key ^ 0xFFFFFFFF);
}

void BotSession::ParseMapFile(char * FileContents, LONG Length)
{
	int i;

	for (i = 0; i < Length; i += 4)
	{
		DWORD value = *(DWORD*)&FileContents[i];

		DWORD Offset = (((value >> 12) & 0xFFF) << 10) + (value & 0xFFF);
		
		value >>= 24;

		Map[Offset] = BYTE(value);
		ChecksumMap[Offset] = BYTE(value);
	}

	for (i = 0; i < 1024; i++)
	{
		Map[i] = 20;
		Map[i + (1023 << 10)] = 20;
		Map[i << 10] = 20;
		Map[1023 + (i << 10)] = 20;
	}
}

void GetFileLength(LONG FileHandle, LONG & FileLength)
{
	_lseek(FileHandle, 0, SEEK_END);

	FileLength = _tell(FileHandle);

	_lseek(FileHandle, 0, SEEK_SET);
}

bool BotSession::LoadMap(const char * MapFilename, LONG Checksum)
{
	// Open map file for information
	LONG fnMap = _open(MapFilename, _O_BINARY, _O_RDONLY);
	if (fnMap == -1)
	{
//		printf("Unable to open map file, requesting download...");

		return false;	// Failed to load
	}

	// Retrieve file size
	LONG FileLength = 0;
	GetFileLength(fnMap, FileLength);

	// Read file contents
	BYTE * MapContents = new BYTE[FileLength];
	_read(fnMap, MapContents, FileLength);

	// Do we need to download the map?
	LONG LocalChecksum = GetFileChecksum(MapContents, FileLength);
	if (LocalChecksum != Checksum)
	{
		delete MapContents;
		_close(fnMap);

		//printf("Map checksum mismatch, requesting download...");

		return false;
	}

	// Get offset to the map portion of the level file
	if ((MapContents[0] == 'B') && (MapContents[1] == 'M'))
	{
		LONG Offset = GetLong(MapContents, 2);

		ParseMapFile((char*)(MapContents + Offset), FileLength - Offset);

		LONG fnTiles = _open("gfx/tileset.tmp", _O_BINARY | _O_CREAT | _O_WRONLY, _S_IWRITE);
		_write(fnTiles, MapContents, Offset);
		_close(fnTiles);

		_tileset * ts = new _tileset(&Window, "gfx/tileset.tmp", 304, 160);
		ts->OpaqueBlit(TileSet.tiles->image, 0, 0, 0);
		delete ts;
	}
	else
	{
		ParseMapFile((char*)(MapContents), FileLength);
	}

	// Close file handle
	delete MapContents;
	_close(fnMap);

	return true;
}

void BotSession::SaveMap(const BYTE * MapFilename, BYTE * MapBytes, LONG MapLength)
{
	BYTE * ResFile = NULL;
	unsigned long ResLength	= (MapLength << 1),
				  status	= 0;

	do
	{
		if (ResFile)
			delete ResFile;

		ResLength += MapLength;
		ResFile = new BYTE[ResLength];

		status = uncompress(ResFile, &ResLength, MapBytes, MapLength);

	} while (status == Z_BUF_ERROR);

	if (status)
	{
		MessageBox(NULL, "Damn uncompress()!!!", "Error:", 0);
		return;
	}

	// Write map
	LONG fnMap = _open((char*)&MapFilename[0], _O_BINARY | _O_CREAT | _O_WRONLY, _S_IWRITE);
	_write(fnMap, ResFile, ResLength);
	_close(fnMap);

	// Get offset to the map portion of the level file
	if ((ResFile[0] == 'B') && (ResFile[1] == 'M'))
	{
		LONG Offset = *(LONG*)&ResFile[2];

		ParseMapFile((char*)(ResFile + Offset), ResLength - Offset);

		LONG fnTiles = _open("gfx/tileset.tmp", _O_BINARY | _O_CREAT | _O_WRONLY, _S_IWRITE);
		_write(fnTiles, ResFile, Offset);
		_close(fnTiles);

		_tileset * ts = new _tileset(&Window, "gfx/tileset.tmp", 304, 160);
		ts->OpaqueBlit(TileSet.tiles->image, 0, 0, 0);
		delete ts;
	}
	else
	{
		ParseMapFile((char*)(ResFile), ResLength);
	}
}

LONG GenPartialExeSum(LONG FileHandle, LONG key, LONG cnt, LONG index)
{
	// Read this part of the EXE
	BYTE * EXEData = new BYTE[cnt];
	_lseek(FileHandle, index, SEEK_SET);
	_read(FileHandle, EXEData, cnt);

	// Generate a checksum for this part
	LONG temp;
	LONG PartialKey = 0;
	LONG offset = 0;

	while(cnt > 4)
	{
		temp = *(LONG*)&EXEData[offset];

		if((cnt & 0x7FF) == 0)
		{
			PartialKey += (temp | key);
		}
		else
		{
			PartialKey = temp ^ key ^ PartialKey;
		}

		cnt -= 3;
		offset += 3;
	}

	// Clean up
	delete EXEData;

	// Return the checksum
	return PartialKey;
}

LONG GenerateEXEChecksum(LONG Key)
{
	LONG fnEXE = _open("subspace.exe", _O_BINARY, _O_RDONLY);

	LONG Checksum = 0;

	Checksum += GenPartialExeSum(fnEXE, Key, 0xD5A0, 0x6540 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0x2480, 0x00 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0x6F70, 0x1C6E0 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0x21E0, 0x42A70 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0xD3C0, 0x301A0 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0x5FE0, 0x2A1B0 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0x1C90, 0x3560 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0x8BE0, 0x13AF0 + 1024);
	Checksum += GenPartialExeSum(fnEXE, Key, 0xE0, 0x5350 + 1024);

	_close(fnEXE);

	return Checksum;
}

LONG GenerateParameterChecksum(LONG Key, LONG * ServerSettings)
{
	LONG Checksum = 0;

	for(LONG i = 0; i < 357; i++)
	{
		Checksum += (ServerSettings[i] ^ Key);
	}

	return Checksum;
}

LONG GenerateLevelChecksum(LONG key, char * mapData)
{
	LONG EAX, ECX, ESI, EDX;
	LONG HighBit = 0;
	LONG cnt;

	LONG original_key = key;

	EAX = key;

	if(EAX & 0x80000000)
		HighBit = 0xFFFFFFFF;

	EAX ^= HighBit;
	EAX -= HighBit;
	EAX &= 0x1F;
	EAX ^= HighBit;

	ECX = EAX - HighBit;

	if(ECX >= 0x400)
		return key;

	EDX = key % 0x1F;

	ESI = (ECX << 0x0A) + (LONG)mapData + EDX;
	EAX = 0x400 - EDX;
	cnt = (0x41F - ECX) >> 5;
	mapData  = (char *)EAX;

	for(; cnt > 0; cnt--, ESI += 0x8000)
	{
		EDX = EAX + ESI;
		ECX = ESI;
		if(!(ESI < EDX))
			continue;
		while(ECX < EDX)
		{
			unsigned char byte = (*((char *)ECX));
			if((byte < 0xA1 || byte == 0xAB) && byte != 0)
				key += original_key ^ byte;

			ECX += 0x1F;
		}
		EAX = (LONG)mapData;
	}

	return key;
}
