/*  Default SSBOT.INI contents:

[Login]
Zone=127.0.0.1:2000
Name=Catid
Pass=+++++

ShipType=Warbird
// ShipType codes:
//  Warbird   WB   1
//  Javelin   JAV  2
//  Spider    SP   3
//  Leviathan LEVI 4
//  Terrier   TERR 5
//  Weasel    WEEE 6
//  Lancaster LANC 7
//  Shark     SK   8
//  Spectator SPEC 9

Arena=
// Arena codes:
//  Arena=          - Let the server decide
//  Arena=@<number> - Specific main arena
//  Arena=<name>    - Specific public arena
//  Arena=#<name>   - Private arena

StaffPassword=
// StaffPassword is not required unless the bot will run games

AccessLevel=Player
// AccessLevel codes:
//  AccessLevel=Player         0 - Behave like a regular player
//  AccessLevel=Moderator      1 - Behave like a Moderator
//  AccessLevel=SuperModerator 2 - Behave like a SuperModerator
//  AccessLevel=SysOp          3 - Behave like a System Operator

[Log]
File=chatter.log
ShowCommands=1
ShowChat=1
ShowKills=1
ShowEvents=1

 */


// Basic container for the INI settings

struct ClientSettings
{
	DWORD IP;
	WORD Port;
	char * Name;
	char * Pass;
	char * Arena;
	ShipClasses ShipType;
	char * StaffPassword;
	AccessLevels AccessLevel;	// TODO
	char * LogPath;				// TODO
	bool ShowCommands;			// TODO
	bool ShowChat;				// TODO
	bool ShowKills;				// TODO
	bool ShowEvents;			// TODO

	ClientSettings()
	{
		Name = NULL;
		Pass = NULL;
		Arena = NULL;
		StaffPassword = NULL;
		LogPath = NULL;

		ReloadSettings();
	}

	~ClientSettings()
	{
		DeleteStrings();
	}

	void DeleteStrings()
	{
		if (Name) delete Name;
		if (Pass) delete Pass;
		if (Arena) delete Arena;
		if (StaffPassword) delete StaffPassword;
		if (LogPath) delete LogPath;
	}

	void ReloadSettings()
	{
		char RetBuf[256];
		DWORD len;

		// Load settings -> Set up server mapping
		char Path[532];
		GetCurrentDirectory(520, Path);
		strcat(Path, "\\SSBOT.INI");

		DeleteStrings();

		len = GetPrivateProfileString("Login", "Zone", "127.0.0.1:2000", RetBuf, 256, Path);

		// Parse the format <IP>:<Port> from a char buffer to a DWORD and WORD.
		{
			char ParseBuf[257];
			DWORD Offset = 0;
			BYTE Char;

			// Step through the char buffer
			for (DWORD i = 0; i < len; i++)
			{
				Char = RetBuf[i];

				if ((Char == ':') || (Char == ','))
				{
					// Time to update the IP
					ParseBuf[Offset] = 0;

					IP = ResolveHostname(ParseBuf);

					Offset = 0;
				}
				else
				{
					ParseBuf[Offset++] = Char;
				}
			}

			// Time to update the Port
			ParseBuf[Offset] = 0;

			Port = atoi(ParseBuf);
		}

		len = GetPrivateProfileString("Login", "Name", "Catid", RetBuf, 256, Path);

		// Trim names longer than 20 characters
		{
			if (len > 20)
				len = 20;

			Name = new char[len + 1];
			Name[len] = 0;
			memcpy(Name, RetBuf, len);
		}

		len = GetPrivateProfileString("Login", "Pass", "+++++", RetBuf, 256, Path);

		// Trim passwords longer than will fit in the password packet field
		{
			if (len > 32)
				len = 32;

			Pass = new char[len + 1];
			Pass[len] = 0;
			memcpy(Pass, RetBuf, len);
		}

		len = GetPrivateProfileString("Login", "ShipType", "Warbird", RetBuf, 256, Path);

		// Extract intended ship type
		{
			bool NotNumeric = true;

			if (len == 1)
			{
				// Ship #
				BYTE Char = RetBuf[0];

				if ((Char > 48) && (Char < 58))
				{
					ShipType = (ShipClasses) (Char - 49);

					NotNumeric = false;
				}
			}

			if (NotNumeric)
			{
				// [Parts of the] full name
				if (MatchStringCI(RetBuf,		"spe", 3))
					ShipType = ssSpectator;
				else if (MatchStringCI(RetBuf,	"j", 1))
					ShipType = ssJavelin;
				else if (MatchStringCI(RetBuf,	"sp", 2))
					ShipType = ssSpider;
				else if (MatchStringCI(RetBuf,	"le", 2))
					ShipType = ssLeviathan;
				else if (MatchStringCI(RetBuf,	"t", 1))
					ShipType = ssTerrier;
				else if (MatchStringCI(RetBuf,	"we", 2))
					ShipType = ssWeasel;
				else if (MatchStringCI(RetBuf,	"l", 1))
					ShipType = ssLancaster;
				else if (MatchStringCI(RetBuf,	"s", 1))
					ShipType = ssShark;
				else
					ShipType = ssWarbird;
			}
		}

		len = GetPrivateProfileString("Login", "Arena", "", RetBuf, 256, Path);

		// Trim arenas longer than will fit in the arena packet field
		{
			if (len > 15)
				len = 15;

			Arena = new char[len + 1];
			Arena[len] = 0;
			memcpy(Arena, RetBuf, len);
		}

		len = GetPrivateProfileString("Login", "StaffPassword", "", RetBuf, 256, Path);

		// Trim passwords longer than will fit in a password packet field
		{
			if (len > 31)
				len = 31;

			StaffPassword = new char[len + 1];
			StaffPassword[len] = 0;
			memcpy(StaffPassword, RetBuf, len);
		}

		len = GetPrivateProfileString("Login", "AccessLevel", "Player", RetBuf, 256, Path);

		// Extract intended access level
		{
			bool NotNumeric = true;

			if (len == 1)
			{
				// Access level #
				BYTE Char = RetBuf[0];

				if ((Char > 47) && (Char < 52))
				{
					ShipType = ShipClasses(Char - 48);

					NotNumeric = false;
				}
			}

			if (NotNumeric)
			{
				// [Parts of the] access level's name
				if (MatchStringCI(RetBuf, "m", 1))
					AccessLevel = ssModerator;
				else if (MatchStringCI(RetBuf, "su", 2))
					AccessLevel = ssSuperModerator;
				else if (MatchStringCI(RetBuf, "s", 1))
					AccessLevel = ssSysOp;
				else
					AccessLevel = ssPlayer;
			}
		}

		len = GetPrivateProfileString("Log", "File", "chatter.log", RetBuf, 256, Path);

		// Copy to a new char buffer
		{
			LogPath = new char[len + 1];
			LogPath[len] = 0;
			memcpy(LogPath, RetBuf, len);
		}

		len = GetPrivateProfileString("Log", "ShowCommands", "1", RetBuf, 256, Path);

		// Either 1 or 0
		{
			ShowCommands = (RetBuf[0] == 1);
		}

		len = GetPrivateProfileString("Log", "ShowChat", "1", RetBuf, 256, Path);

		// Either 1 or 0
		{
			ShowChat = (RetBuf[0] == 1);
		}

		len = GetPrivateProfileString("Log", "ShowKills", "1", RetBuf, 256, Path);

		// Either 1 or 0
		{
			ShowKills = (RetBuf[0] == 1);
		}

		len = GetPrivateProfileString("Log", "ShowEvents", "1", RetBuf, 256, Path);

		// Either 1 or 0
		{
			ShowEvents = (RetBuf[0] == 1);
		}
	}
};
